function mriplot(I)
    s=size(I);
    f=figure(); f.backgroundcolor='1|1|1'; colormap gray;
    subplot(1,3,1), imagesc(rot90(squeeze(I(:,:,fix(s(1)/2)))));
    subplot(1,3,2),  imagesc(rot90(squeeze(I(:,fix(s(2)/2),:))));
    subplot(1,3,3),  imagesc(rot90(squeeze(I(fix(s(3)/3),:,:))));
endfunction
function [jhist,scale]=jointh(A,B,scaled)
    // fast joint histogram of two signals A and B, of the same size
    N=max(max(A(:)),max(B(:)))+1;
   
    s = prod(size(A)); t = 1:s;
    x = A(:)+1;
    y = N*B(:);

    x = x + y;
    x = gsort(x);
    y(1:s-1) = x(2:s);

    z = y - x;
    z(s) = 1;
    z = t(z ~=0);

    y = x(z);
    len = prod(size(z));

    z(2:len) = z(2:len)-z(1:len-1);
    x = z;
    if scaled then
        x=x/s;
    end
    jhist = zeros(N,N);
    jhist(y) = x;
    scale=s;
endfunction

function peaks=findpeaks(A,threshold,ray)
    // detects the local maxima of the matrix A
    if argn(2)<2 then
        threshold=0;
    end
    if argn(2)<3 then
        ray=3;
    end
    [m,n]=size(A);

    //Find point whose (estimated) Hessian is positive
    Dx=A(:,1:$-1)-A(:,2:$);
    Dx1=[Dx, zeros(m,1)];
    Dx2=[zeros(m,1), -Dx];

    Dy=A(1:$-1,:)-A(2:$,:);
    Dy1=[Dy; zeros(1,n)];
    Dy2=[zeros(1,n); -Dy];
    [x,y]=find(Dx1>0 & Dx2>0 & Dy1>0 & Dy2>0 & A>threshold);
    //add corner point that has been excluded
    x=[1,x]; y=[1,y];
    // Select points that are the maximum over its neighborhood
    peaks=[];
    for i=1:length(x)
        i1=max(1,x(i)-ray); i2=min(m,x(i)+ray); 
        i3=max(1,y(i)-ray); i4=min(n,y(i)+ray);
        if abs( A(x(i),y(i))-max(A(i1:i2, i3:i4)) )<1.e-16 then
            peaks=[peaks; x(i), y(i)];
        end
    end

endfunction


function [B,Mx]=gravity_map(A,peaks,threshold)
    //partition of the joint histogram A, by means of gravity areas of the peaks
    s=size(peaks); n=size(A); B=zeros(n(1),n(2)); Mx=B;
    for i =1:s(1)
        X=repmat(1:n(1),n(2),1);
        Dist_sq=1+max((X-peaks(i,2)).^2,(X'-peaks(i,1)).^2);
        mass=A(peaks(i,1),peaks(i,2));
        if i==1 then //reduce field of zero-values
            mass=mass*5.e-3;
        end
        Mass=A*mass;
        Force=Mass./Dist_sq;
        B(Mx<Force & Force>threshold & A>threshold)=i;
        Mx=max(Mx,Force);

    end
    for i =1:s(1)
        B(peaks(i,1),peaks(i,2))=s(1)+1;
    end
endfunction


function [I,varargout]=extract(I1,I2,GM,pks)

    //extract the sub(hyper)matrix of "I1" and "I2" from the gravity map "GM" of the peacks "pks" of the joint histogram "J"
    // the (hyper)matrix I will contain only integer values (max=peack number)
    S=size(I1); L=list();
    I=zeros(prod(S),1); J1=zeros(S(1),S(2),S(3));
    for i=1:size(pks,1)
        [i1,j1]=find(GM==i);
        idx=[i1;j1]; idx=[pks(i,:)',idx]-1; s=size(idx); 
        J=int16(zeros(prod(S),1));

        JJ1=I1(:)>=min(idx(1,:)) & I1(:)<=max(idx(1,:));
        JJ2=I2(:)>=min(idx(2,:)) & I2(:)<=max(idx(2,:));
        J=(JJ1&JJ2);
        I=I+(i-1)*J(:);
        execstr('J'+string(i)+'=matrix(J,S(1),S(2),S(3));');
        execstr('L('+string(i)+')=J'+string(i)+';');

    end
    I=matrix(I,S(1),S(2),S(3));
    varargout=L;
endfunction

function A=intersect(L1,L2,L3)
    s=size(L1)
    B=(L1(:)&L2(:)) | (L1(:)&L3(:)) | (L2(:)&L3(:))
    A=matrix(B,s(1),s(2),s(3))
endfunction

function L=lesion_find(I1,I2,I3,exclude,pkk1,pkk2,pkk3)
    
    
    ray=3.5;
    
    m1=mean(pkk1,1);  m2=mean(pkk2,1);  m3=mean(pkk3,1);
    s1=st_deviation(pkk1,1); s2=st_deviation(pkk2,1);s3=st_deviation(pkk3,1);

    L1=I1>m1(1)-ray*s1(1) & I1<m1(1)+ray*s1(1) & I2>m1(2)-ray*s1(2) & I2<m1(2)+ray*s1(2);
    L2=I1>m2(1)-ray*s2(1) & I1<m2(1)+ray*s2(1) & I3>m2(2)-ray*s2(2) & I3<m2(2)+ray*s2(2);
    L3=I2>m3(1)-ray*s3(1) & I2<m3(1)+ray*s3(1) & I3>m3(2)-ray*s3(2) & I3<m3(2)+ray*s3(2);
    
   
    LL=intersect(L1,L2,L3);
    //LL=L1&L2&L3;

    L=LL& ~exclude ;
endfunction

function J=erode3d(I)
    //erode the 3d binary map I using as morphological filter the 3x3x3 diamond hypermatrix 
    //    0 0 0   0 1 0    0 0 0
    //    0 1 0   1 1 1    0 1 0
    //    0 0 0   0 1 0    0 0 0
    s=size(I);
    I1=I(:); I1=int8(I1);
    
    Iu=[ones(1,s(2),s(3))==1;  I(1:$-1,:,:)];      Iu=int8(Iu(:));
    Id=[I(2:$,:,:);ones(1,s(2),s(3))==1];      Id=int8(Id(:));
    
    Il=[ones(s(1),1,s(3))==1, I(:,1:$-1,:)];       Il=int8(Il(:));
    Ir=[I(:,2:$,:), ones(s(1),1,s(3))==1];     Ir=int8(Ir(:));
    
    If(1:s(1),1:s(2),1)=ones(s(1),s(2),1)==1;
    If(1:s(1),1:s(2),2:s(3))=I(:,:,1:$-1);         If=int8(If(:));
    Ib(1:s(1),1:s(2),1:s(3)-1)=I(:,:,2:$);
    Ib(1:s(1),1:s(2),s(3))=ones(s(1),s(2),1)==1; Ib=int8(Ib(:));
    J=(min(I1,Iu,Id,Il,Ir,If,Ib)==1);
    J=matrix(J,s(1),s(2),s(3));
    
endfunction
function J=dilate3d(I)
    K=~I;
    J=~erode3d(K);
endfunction

function J=opening3d(I)
    K=erode3d(I)
    J=dilate3d(K)
endfunction

function J=closing3d(I)
    K=dilate3d(I)
    J=erode3d(K)
endfunction

function I=ndilate3d(I,k)
    for i=1:k
        I=dilate3d(I)
    end
endfunction
